//Programa para el calculo de la matriz de admitancias
//de red Ybarra en Sistemas de Potencia
function [Y]=Ybarra(a)
 exec(a);                             //Lectura y ejecucin del archivo con los datos de red
 X=ramas;                             //Asignacin de la matriz de ramas del sistema
 [n,m]=size (X);                      //Lectura del tamao de la matriz
 d=max(max(X(:,1)),max(X(:,2)));      //Calculo del numero de barras
 Y=[];    
 for i=1:n                            //Asignacin de los indices a y b, barra de partida y  
  a=X(i,1);                           //barra de llegada, respectivamente
  b=X(i,2);
  if (a==0) then
   Shunt (b,b)=X(i,3)+j*X(i,4);       //Asignacin de las admitancias shunt del sistema
   Shunt (b,b)=(1/Shunt (b,b));       //asociadas a la barra de partida
  else
   if (b==0) then
    Shunt (a,a)=X(i,3)+j*X(i,4);      //Asignacin de las admitancias shunt del sistema
    Shunt (a,a)=(1/Shunt (a,a));      //asociadas a la barra de llegada
   else 
    Shunt (a,b)=j*X(i,5);
    Shunt (b,a)=j*X(i,5);             //Asociacin de los valores de impedancia de red
    Z(a,b)=X(i,3)+j*X(i,4);
    if (Z(a,b)<>0)
     Y(a,b)=(-1)*(1/Z(a,b));          //Calculo de las admitancias de red
     Y(b,a)=(-1)*(1/Z(a,b));
    else
     Y(a,b)=Z(a,b);                   //Validacin en caso de encontrar impedancias cero
     Y(b,a)=Z(a,b);
    end
   end 
  end 
 end
 for i=1:d
  Y(i,i)=(-1)*sum(Y(i,:))+sum(Shunt(i,:)); //Asociacin final con los valores shunt
 end
printf ("LA MATRIZ DE ADMITANCIAS DE RED ES:\n");
endfunction

//=========================================================================
//=========================================================================

//Programa para el calculo de la matriz de impedancias de red
//Zbarra en Sistemas de Potencia

function [ZZ]=Zbarra(a)
//exec(a);                             //Lectura y ejecucin del archivo con los datos de red
//X=ramas;                             //Asignacin de la matriz de ramas del sistema
X=a;
[a,b]=size (X);                      //Lectura del tamao de la matriz de datos
n=max(max(X(:,1)),max(X(:,2)));      //Calculo del numero de barras del sistema 
e=(-2)*ones(n,1);
Suma=zeros(n,1);                     //Inicializacin de variables
Z=zeros(n+1,n+1);
K=0;
for i=1:n
 for k=1:a
  if (X(k,1)==i) then                //Asociacin de valores de admitancia shunt
   Suma(i)=X(i,5)+Suma(i);
  end
 end
 if (Suma(i)<>0) then 
  Suma(i)=j*(1/Suma(i));             //Traspaso de admitancia shunt a valores de impedancia
  e(i)=-1;
 end
end

if (nnz(Suma)==0) then
 if (nnz(X(:,1))==a) then            //Asignacin del nodo de referencia del sistema
  if (nnz(X(:,2))==a) then
   refer=X(1,1);
  else                               //Por defecto se asigna como referencia al nodo cero
   refer=0;
  end
 else                                //En caso contrario se asigna como referencia el primer
  refer=0;                           //nodo encontrado en la lectura de datos, por lo general,
 end                                 //es el nodo numero uno
else 
 refer=0;
end

for i=1:a
 if (X(i,1)==refer)
  if (e(X(i,2))==-1)                 //Asignacin de los posibles casos para la formacin de 
   caso=3;                           //la matriz de impedancias
  else
   caso=1;                           //Los casos son los siguientes: 
   e(X(i,2))=-1;
  end                                //caso1: aadir una rama nueva al nodo de referencia
 else                               
  if (X(i,2)==refer)                 //caso2: aadir una nueva rama a una barra existente
   if (e(X(i,1))==-1)
    caso=3;                          //caso3: aadir una rama entre una barra existente y el nodo de referencia
   else
    caso=1;                          //caso4: aadir una rama entre dos barras existentes
    e(X(i,1))=-1;
   end 
  else
   if (e(X(i,1))==-1)
    if (e(X(i,2))==-1)
     caso=4;
    else
     caso=2;
     e(X(i,2))=-1;
    end 
   else
    if (X(i,2)==-1)
     caso=2;
     e(X(i,1))=-1;
    else 
     printf("Caso no establecido\n"); //En caso de no identificar ninguno de los casos preestablecidos
     caso=-1;
    end
   end
  end
 end
 
 select caso                          //Ejecucin de cada uno de los casos y modificacin de la Zbarra
  
  case 1 then
   Z(K+1,K+1)=X(i,3)+j*X(i,4);
   for l=1:K
    Z(K+1,l)=0;
    Z(l,K+1)=0;
   end
   K=K+1;
   
  case 2 then
   Z(K+1,K+1)=Z(K,K)+X(i,3)+j*X(i,4);
   for l=1:K
    Z(K+1,l)=Z(K,l);
    Z(l,K+1)=Z(l,K);
   end
   K=K+1;
   
  case 3 then
   Z(K+1,K+1)=Z(K,K)+X(i,3)+j*X(i,4);
   for l=1:K
    Z(K+1,l)=Z(K,l);
    Z(l,K+1)=Z(l,K);
   end
   for l=1:K
    for m=1:K
     Z(l,m)=Z(l,m)-((Z(l,K+1)*Z(K+1,m))/Z(K+1,K+1));
    end
   end
   for l=1:K+1
    Z(K+1,l)=0;
    Z(l,K+1)=0;
   end 
  
  case 4 then
   Z(K+1,K+1)=Z(X(i,1),X(i,1))+Z(X(i,2),X(i,2))-2*Z(X(i,1),X(i,2))+(X(i,3)+j*X(i,4));
   for l=1:K
    Z(K+1,l)=Z(X(i,1),l)-Z(X(i,2),l);
    Z(l,K+1)=Z(l,X(i,1))-Z(l,X(i,2));
   end
   for l=1:K
    for m=1:K
     Z(l,m)=Z(l,m)-((Z(l,K+1)*Z(K+1,m))/Z(K+1,K+1));
    end
   end
   for l=1:K+1
    Z(K+1,l)=0;
    Z(l,K+1)=0;
   end 
  
  else
   printf ("CASO NO ENCONTRADO\n");
 end 
end

ZZ=zeros(n,n);
for l=1:n
 for m=1:n
  ZZ(l,m)=Z(l,m);                  //Reajustamiento de la dimensin de la matriz de impedancias
 end
end 
//printf ("LA MATRIZ DE IMPEDANCIAS DE RED ES:\n");
endfunction  

